/**************************************/
/*	 AVR Mega8 			              */
/**************************************/

#define F_CPU 8000000UL
//----------------------------------------------------------
#include <avr/io.h> 				
#include <avr/interrupt.h> 
//----------------------------------------------------------
#include "Timer0.h"			// Main() - Zeitschleifen
#include "lcd_routines.h"	// Pinbelegung LCD ber defines in lcd-routines.h einstellen
#include "avr/pgmspace.h"
#include "stdlib.h"

#include "global.h"
#include "rfm12.h"
#include "taster2.h"
#include "LED.h"
//#include <util/delay.h>

      
//------------------------------------- Variablen -----------
//volatile unsigned char blink;
//static int i = 0;
 static int counter = 0;
 static int menu = 0;
 static int min = 0;
 static int sek = 0;

 uint8_t leer = 0;
 uint8_t tast1 = 0;
 uint8_t tast2 = 0;
 uint8_t tast3 = 0;


//------------ Application  init ---------------------------

void Application_init (void)
{	
	Timer0_init ();	  // Timer initialisieren
	OSCCAL = 0xAF;	  // Kalibrierung;
	lcd_init();		  // Initialisierung des LCD
	sei ();			  // Enable global Interrupt

	rf12_init();					// ein paar Register setzen (z.B. CLK auf 10MHz)
	rf12_setfreq(433.92);			// Sende/Empfangsfrequenz auf 433,92MHz einstellen
	rf12_setbandwidth(4, 0, 4);		//5,1,4; 200kHz Bandbreite, -6dB Verstrkung, DRSSI threshold: -79dBm 
	rf12_setbaud(4800);				// 19200 baud
	rf12_setpower(0, 6);			// 1mW Ausgangangsleistung, 120k

	
//-------------------------- LEDs --------------------------
	DDRB |= (1<<LED1) | (1<<LED2) | (1<<LED3);
		
	LED_aus(LED1);
	LED_aus(LED2);
	LED_aus(LED3);

//------------------------- Taster -------------------------
	DDRB  &= ~(1<<TASTER1);
	DDRB  &= ~(1<<TASTER2);
	DDRB  &= ~(1<<TASTER3);
	
//------------------------- Summer -------------------------
		//DDRC &= ~(1<<PC4); 
		DDRC = 0b00010000;	// PIN Summer auf Ausgang

//----------------- LCD 1. Anzeige -----------------------
		PORTD |= 0b00010000;  // LCD Beleuchtung ein
		lcd_setcursor(0, 1 );	
    	lcd_string("Hallo" );

//------------------------- RFM12 TX RX -------------------------


//----------------- LCD eigenes Zeichen ------------------
	uint8_t chrdata1[8] = { // 
        0b00001110,		//  XXX
        0b00001010,		//  X X
        0b00001110,     //  XXX
        0b00000000,     
        0b00000000,     
        0b00000000,     
        0b00000000,
        0b00000000
        };

	lcd_generatechar(LCD_GC_CHAR1, chrdata1);	
	lcd_setcursor(14,1);
    LCD_Data(LCD_GC_CHAR1);	
}



//------------- Zeitschleife 10ms ----------------------------
void Zeitschleife_10ms ()
{		
//--------------- Tasterabfrage -------------------------

		if (taster1_druck() && tast1 == 0)
		{	
			tast1 = 1;
			LED_ein(LED1);
			PORTD |= 0b00000000;	 // LCD Beleuchtung aus
			//menu = 0;
		}

		else if (taster2_druck() && tast2 == 0)
		{
			tast2 = 1;
			PORTD |= 0b00010000;  // LCD Beleuchtung ein
			//menu = 0;
			menu++;
			LED_toggle(LED2);
			PORTC &= ~(1<<PC4);	// Summer ein
		}

		else if (taster3_druck() && tast3 == 0)
		{
			tast3 = 1;
			LED_ein(LED1);
			PORTD |= 0b00010000;  // LCD Beleuchtung ein
			//menu = 0;
		}	

						
}
//------------- Zeitschleife 100ms ---------------------------
void Zeitschleife_100ms ()
{	
		  lcd_setcursor(9, 1 );		
		  LCD_PrintZahl16(counter);
		  lcd_setcursor(15, 1 );
		  lcd_string ("C");
		
	//_____________ Summer _______________________________
		PORTC |= (1<<PC4); // Summer aus nach <= 100ms
		

	//_____________ Menu LCD _____________________________
		
		switch(menu)
	{
		case 0:
		lcd_setcursor(0, 2 );		
		if (leer == 1) 
		{
			lcd_string ("                ");
			leer = 0;
		}
		break;

		case 1:
			lcd_setcursor(0, 2 );		
			lcd_string ("Hauptmenu    ");
		break;

		case 2:
			lcd_setcursor(0, 2 );		
			lcd_string ("1. Untermenu:");
		break;

		case 3:
			lcd_setcursor(0, 2 );		
			lcd_string ("2. Untermenu:");
			leer = 1;
		break;

			
	}
		
	if (menu == 4) { menu = 0;};
				
}
//------------- Zeitschleife 500ms ---------------------------
void Zeitschleife_500ms ()
{
	
}
//------------- Zeitschleife 1s ---------------------------
void Zeitschleife_1s (void)
{		
		tast1 = 0;	//
		tast2 = 0;	// Tasterentprellung
		tast3 = 0;	//

		counter++;
		sek++;

		if (counter % 60 ==0) {min++;};
		if (sek == 60) {sek = 0;};
	
		if (counter % 5 == 0) LED_ein(LED3); //	LED3 blinkt alle 5s
			else LED_aus(LED3);				 //

			
		//____________ RFM12 RX TX _________________________

		if (counter % 5 == 0) {receive();}; 	// Alle 5 Sekunden Empfangen
												// Verrbeitung Daten in rfm12.c Funktion: receive()
}

//------------- Zeitschleife IDLE ----------------------------

void Zeitschleife_Idle (void)
{				
}


